﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

# <rtc-template block="description">
"""
 @file PoseDetector.py
 @brief ModuleDescription
 @date $Date$


"""
# </rtc-template>

import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist
import Img

# Import other libraries
import numpy as np
import cv2

import time
import ast

# Openpose libraries donot need any make or installation, Just download it from the github and copy it to your component folder.
# https://github.com/ZheC/tf-pose-estimation
# Add path of Openpose and import Openpose libraries
import os
sys.path.append(os.path.join(os.path.dirname(__file__), 'tfpose/src/'))

import common
from estimator import TfPoseEstimator
from networks import get_graph_path, model_wh

from lifting.prob_model import Prob3dPose
from lifting.draw import plot_pose

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
posedetector_spec = ["implementation_id", "PoseDetector", 
         "type_name",         "PoseDetector", 
         "description",       "ModuleDescription", 
         "version",           "1.0.0", 
         "vendor",            "VenderName", 
         "category",          "ImageProcessiong", 
         "activity_type",     "STATIC", 
         "max_instance",      "1", 
         "language",          "Python", 
         "lang_type",         "SCRIPT",
         ""]
# </rtc-template>

# <rtc-template block="component_description">
##
# @class PoseDetector
# @brief ModuleDescription
# 
# 
# </rtc-template>
class PoseDetector(OpenRTM_aist.DataFlowComponentBase):
	
    ##
    # @brief constructor
    # @param manager Maneger Object
    # 
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_camImg = OpenRTM_aist.instantiateDataType(Img.ImageData)
        """
        """
        self._camImgIn = OpenRTM_aist.InPort("camImg", self._d_camImg)
        self._d_posePos = OpenRTM_aist.instantiateDataType(RTC.TimedDoubleSeq)
        """
        """
        self._posePosOut = OpenRTM_aist.OutPort("posePos", self._d_posePos)


		


        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
		
        # </rtc-template>


		 
    ##
    #
    # The initialize action (on CREATED->ALIVE transition)
    # 
    # @return RTC::ReturnCode_t
    # 
    #
    def onInitialize(self):
        # Bind variables and configuration variable
		
        # Set InPort buffers
        self.addInPort("camImg",self._camImgIn)
		
        # Set OutPort buffers
        self.addOutPort("posePos",self._posePosOut)
		
        # Set service provider to Ports
		
        # Set service consumers to Ports
		
        # Set CORBA Service Ports
		
        return RTC.RTC_OK
	
    ###
    ## 
    ## The finalize action (on ALIVE->END transition)
    ## 
    ## @return RTC::ReturnCode_t
    #
    ## 
    #def onFinalize(self):
    #

    #    return RTC.RTC_OK
	
    ###
    ##
    ## The startup action when ExecutionContext startup
    ## 
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK
	
    ###
    ##
    ## The shutdown action when ExecutionContext stop
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK
	
    ###
    ##
    ## The activated action (Active state entry action)
    ##
    ## @param ec_id target ExecutionContext Id
    ## 
    ## @return RTC::ReturnCode_t
    ##
    ##
    def onActivated(self, ec_id):
        # Read model
        # Option :cmu (good accuracy but heavy, need download), mobilenet_thin (bad accuracy but fast, dont need download)
        self.w, self.h = model_wh('mobilenet_thin_432x368') # 'cmu_640x480 / cmu_640x360 / mobilenet_thin_432x368'
        
        # The path to the model.
        # However, there is an unsolved error when using cmu model. 
        self.e = TfPoseEstimator('tfpose/models/graph/mobilenet_thin_432x368/graph_opt.pb', target_size=(self.w, self.h))
        
        return RTC.RTC_OK
	
    ###
    ##
    ## The deactivated action (Active state exit action)
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onDeactivated(self, ec_id):
    #
    #    return RTC.RTC_OK
	
    ##
    #
    # The execution action that is invoked periodically
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onExecute(self, ec_id):
        # receive image from other component
        self._d_camImg = self._camImgIn.read()

        # initialize the pose data
        pose = [0 for j in range(36)]

        if len(self._d_camImg.raw_data) > 0:
            # change image type from img:ImageData to numpy
            oriImg = np.frombuffer(self._d_camImg.raw_data, np.uint8)
            oriImg = np.reshape(oriImg, (self._d_camImg.height, self._d_camImg.width, 3))
            
            # human and human joint detection
            humans = self.e.inference(oriImg)
            # draw
            viewImg = TfPoseEstimator.draw_humans(oriImg, humans, imgcopy=False)

            # joint number
            #0:Nose 1:Neck 2:RShoulder 3:RElbow 4:RWrist 5:LShoulder 6:LElbow 7:LWrist
            #8:RHip 9:RKnee 10:RAnkle 11:LHip 12:LKnee 13:LAnkle 14:REye 15:LEye 16:REar 17:LEar 18:Background

            if len(humans) > 0:
                for i in range(18):
                    if i in humans[0].body_parts.keys():
                        # save joint data as a list. 
                        # e.g., the (x, y) of RElbow is (pose[3*2], pose[3*2+1])
                        # e.g., the (x, y) of LWrist is (pose[7*2], pose[7*2+1])
                        pose[i*2] = int( humans[0].body_parts[i].x * self._d_camImg.width )
                        pose[i*2+1] = int( humans[0].body_parts[i].y * self._d_camImg.height )
            
            # view
            cv2.imshow('camImg', viewImg)
            cv2.waitKey(1)

        # send pose data
        OpenRTM_aist.setTimestamp(self._d_posePos)
        self._d_posePos.data = pose
        print('sent data:  ', self._d_posePos)
        self._posePosOut.write()
    
        return RTC.RTC_OK
	
    ###
    ##
    ## The aborting action when main logic error occurred.
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onAborting(self, ec_id):
    #
    #    return RTC.RTC_OK
	
    ###
    ##
    ## The error action in ERROR state
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onError(self, ec_id):
    #
    #    return RTC.RTC_OK
	
    ###
    ##
    ## The reset action that is invoked resetting
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK
	
    ###
    ##
    ## The state update action that is invoked after onExecute() action
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##

    ##
    #def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK
	
    ###
    ##
    ## The action that is invoked when execution context's rate is changed
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK
	



def PoseDetectorInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=posedetector_spec)
    manager.registerFactory(profile,
                            PoseDetector,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    PoseDetectorInit(manager)

    # create instance_name option for createComponent()
    instance_name = [i for i in sys.argv if "--instance_name=" in i]
    if instance_name:
        args = instance_name[0].replace("--", "?")
    else:
        args = ""
  
    # Create a component
    comp = manager.createComponent("PoseDetector" + args)

def main():
    # remove --instance_name= option
    argv = [i for i in sys.argv if not "--instance_name=" in i]
    # Initialize manager
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()

